# -*- coding: utf-8 -*-
#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:coding=utf-8
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
import concrete.context.ttypes
import concrete.metadata.ttypes
import concrete.language.ttypes
import concrete.structure.ttypes
import concrete.entities.ttypes
import concrete.situations.ttypes
import concrete.exceptions.ttypes
import concrete.email.ttypes
import concrete.twitter.ttypes
import concrete.audio.ttypes
import concrete.communication.ttypes
import concrete.services.ttypes

from thrift.transport import TTransport
all_structs = []
fix_spec(all_structs)
del all_structs
